# -*- coding: utf-8 -*-

"""
file: pyrduino.py (client)
author: Álvaro Muñoz - alvaromm@correo.ugr.es
date: Feb 2014
description:

    This module is based on Python-Arduino-Proto-API-v2 to interact
    with Arduino board by serial port. It provides several functions
    to play with servo, buzzer, leds and any digital or analog
    device attached to the board. 

All features marked with [A] already existed in Python-Arduino-Proto-API-v2
project, but most of them have been modified.

Copyright (C) 2014  Álvaro Muñoz

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or any 
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see http://www.gnu.org/licenses/ .

    Feel free to mail me if bugs or new ideas!

"""

#-*- coding: utf-8 -*-

import sys
import os

try:
    import serial
except AssertionError:
    print "pySerial not installed. Exiting..."
    sys.exit()
    

class Arduino(object):
    
    """ Abstracction of Arduino board """   
    
    SERVO_USED_PINS = [] 
    
    def __init__(self, port=None, baud=9600):
        """ Init board object """
        self.parent = self
        self.baud = baud
        self.config()
        self.port = port
        self.connstate = self.connect()
        
    def connect(self):
        """ Serial connection with Arduino """
        try:
            print "Trying to connect with Arduino on port %s..." % (self.port),
            self.ser = serial.Serial(self.port, self.baud)
            self.ser.write('99')
            print "[OK] connected."
            return 1
        except IOError:
            print "not found."
            return 0
            
    def config(self):
        """ Get user defined PINs from board.conf configuration file """
        if os.path.isfile('board.conf'):
            conf = []
            f = open("board.conf",'r')
            for line in map(lambda x: x.strip(), f.readlines()):
                if line.strip() and not '--' in line:
                    conf.append(line)
            f.close()
            self.OUTPUT_PINS = map(int,conf[0].split(','))
            self.PWM_PINS = map(int,conf[1].split(','))
            print "Config file: 'board.conf'"
            print "  - OUTPUT PINS:", self.OUTPUT_PINS
            print "  - PWM PINS:", self.PWM_PINS
        else:
            self.OUTPUT_PINS = list(range(2,14))
            self.PWM_PINS = [3,5,6,9,10,11]
            print "Config file not found. Using Arduino UNO by default."

    def close(self):
        """ [A] Close serial connection """
        self.ser.close()
        
    def __str__(self):
        """ [A] Returns the board state as string """
        if self.connstate == 1:
            return "Arduino connected on port %s. Baudrate: %s." \
            % (self.port,self.baud) 
        else:
            return "Arduino not connected."
            
    def sendData(self, data):
        """ [A] Send communication protocol """
        while(self.getData()[0] != "w"):
            pass
        self.ser.write(str(data))        
            
    def getData(self):
        """ [A] Receive communication protocol """
        return self.ser.readline().rstrip('\n')
        
    def writeDigital(self, pin, value):
        """ [A] Write a digital value """
        value = str(value)
        if value == '1':
            self.sendData(1)
        elif value == '0':
            self.sendData(0)
        self.sendData(pin)   
        
    def readDigital(self, pin):
        """ [A] Read a digital value """
        self.sendData(2)
        self.sendData(pin)
        if self.getData()[0] == '1': 
            return 1
        return 0
        
    def writeAnalog(self, pin, value):
        """ [A] Write an analog value. 
            Only works with PWM pins """
        if pin in self.PWM_PINS:
            self.sendData(3)
            self.sendData(pin)
            self.sendData(value)
        else:
            print "Pin is not PWM."
        
    def readAnalog(self, pin):
        """ [A] Read an analog value.
            Use only with analog pins """
        self.sendData(4)
        self.sendData(pin)
        return self.getData()
             
    def setOutputPins(self, pins=None):
        """ [A] Set pin (or list) as output.
            By default all pins in arduino.conf """
        if pins==None:
            pins = self.OUTPUT_PINS
        if type(pins) == type(int()):
            pins = [pins]
        self.sendData(len(pins))        
        for pin in pins:
            self.sendData(pin)
    
    def ledOn(self, pins):
        """ Turn on a LED (or list) """
        if type(pins) == type(int()):
            pins = [pins]
        for pin in pins:
            self.writeDigital(pin, 1)
        
    def ledOff(self, pins):
        """ Turns off a LED (or list) """
        if type(pins) == type(int()):
            pins = [pins]
        for pin in pins:
            self.writeDigital(pin, 0)
            
    def readLight(self, pin):
        """ Get light from a standard photosensitive sensor.
            Returns values from 0 to 1 """
        return float(self.readAnalog(pin))/1000
        
    def readTemp(self, pin):
        """ Get temp from a standard temperature sensor.
            Return values in degrees """
        return float(self.readAnalog(pin))/10
        
    def setServo(self, pin):
        """ Set a servo to use it.
            Only works with PWM pins """
        if pin in self.PWM_PINS:
            if pin not in self.SERVO_USED_PINS:
                self.sendData(6)
                self.sendData(pin)     
                self.SERVO_USED_PINS.append(pin)
            else:
                print "Servo already set on pin %s." % pin
        else:
            print "Servo must be in a PWM pin."
        
    def posServo(self, pin, deg):
        """ Places the servo in the desired angle.
            Clockwise in growing angles """
        if pin in self.SERVO_USED_PINS:
            self.sendData(5)
            self.sendData(pin)
            self.sendData(deg)
        else:
            print "Servo must be set."
    
    def beep(self, pin, freq=700, duration=100):
        """ Plays a beep on buzzer 
            freq in Hz, duration in ms """
        self.sendData(7)
        self.sendData(pin)
        self.sendData(freq)
        self.sendData(duration)
        